#ifndef __JACU_H
#define __JACU_H

#if defined(_MSC_VER) && (_MSC_VER < 1400)// VC++6 ĺ   ȸ
#pragma warning (disable: 4786)
#endif

#include <iostream>
#include <string>
#include <map>

using std::string;
using std::map;

#define JACU_ASSERT(cond) jacu::assertTrue(cond, #cond, __FILE__, __LINE__)

#define JACU_ASSERT_EQUALS(expected, actual) jacu::assertEquals(expected, \
actual, __FILE__, __LINE__)

#define JACU_TEST(suite, func) \
namespace suite \
{ \
		void func(); \
} \
jacu::Register Reg_##suite##_##func(#suite, #func, &suite::func); \
void suite::func()
//end #define JACU_TEST

#define JACU_FIXTURE(suite)  namespace suite
#define JACU_SETUP(suite) JACU_TEST(suite, setUp)
#define JACU_TEARDOWN(suite) JACU_TEST(suite, tearDown)

namespace jacu
{
	class BaseReporter;
	
	void runAll(BaseReporter*);
	void runAll();
	void runSuite(const string& suiteName, BaseReporter*);
	void runSuite(const string& suiteName);	

	void assertTrue(bool condition, const char * cond_str,
		const char * file, int line);
	template<typename T>
	void assertEquals(T expected, T actual, const char * file, int line);

	struct BaseReporter
	{
		virtual ~BaseReporter() {};
		template<typename T>
		void addFailedEqualityAssertion(T expected, T actual,
			const char * file, int line);
		void addFailedAssertion(const char* cond_str, const char *file,
								int line);
		void startSuite(const string& suitName);
		void endSuite(const string& suitName);		
	};

	typedef void (*TFP)();

	class CurrentTest
	{
		CurrentTest(){}
		CurrentTest(const CurrentTest&);
		CurrentTest& operator=(const CurrentTest&);
		BaseReporter* theReporterPtr;
	public:
		~CurrentTest() {};
		static CurrentTest* Instance(); 	//  ü  
		void setReporter(BaseReporter *reporter) { theReporterPtr = reporter; }
		void addFailedAssertion(const char* cond_str, const char *file,
			int line);
		template<typename T>
		void addFailedEqualityAssertion(T expected, T actual,
			const char * file, int line);
		BaseReporter* Reporter() { return theReporterPtr;}
		string SuiteName;
		string TestName;
	};

	class Register
	{
	public:
		Register(const char* suite_name, const char* func_name,
			TFP fp);
	};

/***********************************************
 *************    ø     *************
 ***********************************************/

	template<typename T>
	void CurrentTest::addFailedEqualityAssertion(
		T expected, T actual, const char * file, int line
	)
	{
		theReporterPtr->addFailedEqualityAssertion(expected, actual,
			file, line);
	}

	template<typename T>
	void assertEquals(T expected, T actual, const char * file, int line)
	{
		if( expected != actual)
		{
			CurrentTest::Instance()->addFailedEqualityAssertion(expected,
				actual, file, line);
		}
	}

	template<typename T>
	void BaseReporter::addFailedEqualityAssertion(T expected, T actual,
			const char * file, int line)
	{
		std::cerr << CurrentTest::Instance()->SuiteName << "::"
			<< CurrentTest::Instance()->TestName
			<< " FAILED\n"
			<< expected << " expected, " << actual << " actual\n"
			<< file << ": " << line << "\n\n";
	}
}

#endif //__JACU_H
