#include "../include/jacu.h"
#include "jacu_priv.h"
using namespace jacu;

using std::string;
using std::cerr;
using std::cout;
using std::endl;

// jacu Լ 
void jacu::runSuite(const string& suiteName, BaseReporter* reporterPtr)
{
    CurrentTest::Instance()->setReporter(reporterPtr);
    TestManager::Instance()->runSuite(suiteName);
}

void jacu::runSuite(const string& suiteName)
{
	BaseReporter rpt;
	jacu::runSuite(suiteName, &rpt);
}

void jacu::runAll(BaseReporter* reporterPtr)
{
    CurrentTest::Instance()->setReporter(reporterPtr);
    TestManager::Instance()->runAll();
}

void jacu::runAll()
{
	BaseReporter rpt;	
	runAll(&rpt);
}

void jacu::assertTrue(bool condition, const char * cond_str,
                      const char * file, int line)
{
    if (!condition)
    {
        CurrentTest::Instance()->addFailedAssertion(
            cond_str,
            file,
            line
        );
    }
}

// Register 
Register::Register(const char* suiteName, const char* funcName,
                   TFP fp)
{
    TestManager::Instance()->add(suiteName, funcName, fp);
}

// CurrentTest 
CurrentTest* CurrentTest::Instance()
{
    static CurrentTest inst;
    return &inst;
}

void CurrentTest::addFailedAssertion(const char* cond_str, const char *file,
                                     int line)
{
	theReporterPtr->addFailedAssertion(cond_str, file, line);
}

void BaseReporter::addFailedAssertion(const char* cond_str, const char *file,
		int line)
{
	std::cerr << CurrentTest::Instance()->SuiteName << "::"
	<< CurrentTest::Instance()->TestName
	<< " FAILED\n"
	<< "Failed assertion: " << cond_str << "\n"
	<< file << ": " << line << "\n\n";
}

void BaseReporter::startSuite(const string& suitName)
{
	cout << "Testing " << suitName << " ... \n";
}

void BaseReporter::endSuite(const string& suitName)
{
	cout << "... " << suitName << " done. \n";
}
